/*****************************************************************
 *
 *  Cistream.h                      XMM Science Analysis Subsystem
 *
 *  An istream using a CommentBuf
 *
 *  (c) 1998 ESA  -  Written by Jon Brumfitt, Aurora
 *                -  Exported by Jorgo Bakker, BSC to layout package
 *
 *  $Id: Cistream.h,v 1.1 2000/04/14 09:39:50 jbakker Exp $
 *
 *****************************************************************/

#ifndef CISTREAM_H
#define CISTREAM_H

#include <iostream.h>          // C++ headers
#include <streambuf.h>
#include <string>

#include "CommentBuf.h"


/*****************************************************************
 *  Class Cistream
 *
 *  An istream using a CommentBuf
 *
 *  A Cistream is used like a normal istream, except that it strips
 *  comments and provides member functions for getting the current
 *  line and its line number. For example:
 *
 *      ifstream iss("data");
 *      Cistream is(iss.rdbuf());
 *
 *****************************************************************/

class Cistream : public istream
{
public:

    // Construct a Cistream using an existing streambuf
    Cistream(streambuf* sb) : istream(&_cbuf),_cbuf(sb) {}

    // Return number of the current line
    int lineNumber() const { return _cbuf.lineNumber(); }

    // Return the current line
    virtual string line() const { return _cbuf.line(); }

protected:
    
    // Member data
    CommentBuf _cbuf;
};

#endif
