/*****************************************************************
 *
 *  CommentBuf.cc                   XMM Science Analysis Subsystem
 *
 *  streambuf with comment stripping and line counting
 *
 *  (c) 1998 ESA  -  Written by Jon Brumfitt, Aurora
 *                -  Exported by Jorgo Bakker, BSC to layout package
 *
 *  $Id: CommentBuf.cc,v 1.1 2000/04/14 09:39:50 jbakker Exp $
 *
 *****************************************************************/


#include <iostream.h>          // C++ headers
#include <fstream.h>
#include <streambuf.h>
#include <string>
#include <strstream.h>
#include <sys/types.h>

#include "Cistream.h"          // Application headers
#include "CommentBuf.h"


//================================================================
//  Class CommentBuf
//================================================================

//------------------------------
//  Constructors and destructors
//------------------------------

/* Constructor */

CommentBuf::CommentBuf(streambuf *buf, bool autoDel)
{
    /* Initial buffer size. The buffer expands dynamically */

    const int bufSize = 100;

   
    /* Initialise member data */
    
    _lineNumber = 0; 
    _lineLength = -1; 
    _autoDel    = autoDel;
    _sbuf       = buf;

    
    /* Create a new buffer */

    char* buffer = new char[bufSize];
    setb(buffer, buffer + bufSize, 1);
}


/* Destructor */

CommentBuf::~CommentBuf()
{
    /*
     *  There is no need to delete "buffer", since it is
     *  deleted automatically, when _sbuf is deleted.
     */

    if (_autoDel)
        delete _sbuf;
}


//------------------------------
//  Accessors
//------------------------------

/* Return current line */

string CommentBuf::line() const
{
    long len = _lineLength;
    if (len > 0)
        len--;

    return string(base(), len);
}


/* Return current line number */

int CommentBuf::lineNumber() const
{
    return _lineNumber;
}


//------------------------------
//  Buffer manipulation
//------------------------------

/* Buffer underflow - Fetch more data */

int CommentBuf::underflow()
{
    char *ptr = base();              // Start of buffer
    streambuf *sb = _sbuf;           // Pointer to streambuf


    /* Copy a line from streambuf to buffer */

    int ch;
    for (;;) {
        ch = sb->sbumpc();          // Get next char from streambuf

	if (ch == EOF)              // End of file?
	    break;

	if (ptr == ebuf())          // end of buffer?
	    ptr = enlargeBuffer();
		 
        if (ch == '#')              // Comment?
	{
	    while (((ch = sb->sbumpc()) != '\n') && (ch != EOF));

	    if (ptr == ebuf())          // end of buffer?
	        ptr = enlargeBuffer();

            if (ch == '\n')
	        *ptr++ = ch;
	    break;
	}

	*ptr++ = ch;                // Copy char to buffer

	if (ch == '\n')             // End of line?
	    break;
    }

    _lineLength = ptr - base();

    if ((ch != EOF) || (_lineLength > 0))
        _lineNumber++;

    setg(base(), base(), ptr);

    return (_lineLength == 0) ? EOF : base()[0];
}


/*
 *  Enlarge the buffer and return pointer to previous end point.
 *  Hence the buffer dynamically resizes to accomodate the longest line.
 */

char* CommentBuf::enlargeBuffer()
{
    /* Allocate a larger buffer and copy old contents */

    int size = ebuf() - base();
    char *new_buffer = new char[size * 2];
    memcpy(new_buffer, base(), size);

    /*
     *  Set this as the buffer to use.
     *  setb() automatically deletes the old buffer.
     */
    
    setb(new_buffer, new_buffer + 2 * size, 1);

    return new_buffer + size;
}

