/****************************************************************************
 *
 *  testCistream.cc
 *
 *  Test program for Cistream, Cifstream & Cstrstream classes
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: testCistream.cc,v 1.1 2000/04/14 09:39:51 jbakker Exp $
 *
 *****************************************************************************/

#include "Cistream.h"
#include "Cistream.h"
#include "Cistrstream.h"
#include "Cifstream.h"

#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <string>

#include <assert.h>

int main()
{
    /*** Normal istrstream (for reference): normal read to EOF ***/
    {
        cout << "istrstream: normal read to EOF" << endl;

	istrstream is("12"
		       "34\n"
		       "45\n"
		       "xyz\n"
		       "ab cd\n");

	cout << "status = " <<is.good() << is.bad() << is.fail() << is.eof() << endl;
	assert(is.good());
	assert(!is.bad());
	assert(!is.fail());
	assert(!is.eof());

	string n;
	while (is >> n)
	{
	    cout << "status = " <<is.good() << is.bad() << is.fail() << is.eof() << endl;
	    assert(is.good());
	    assert(!is.bad());
	    assert(!is.fail());
	    assert(!is.eof());
	}
	cout << "status = " <<is.good() << is.bad() << is.fail() << is.eof() << endl;

	assert(!is.good());
	assert(!is.bad());
	assert(is.fail());
	assert(is.eof());
	assert(!is);
	assert((void*)is == 0);

	cout << endl;
    }

    /*** Normal istrstream (for reference): read with error ***/
    {
        cout << "istrstream: read with error" << endl;

	istrstream is("12"
		       "34\n"
		       "45\n"
		       "xyz\n"
		       "ab cd\n");

	cout << "status = " << is.good() << is.bad() << is.fail() << is.eof() << endl;
	assert(is.good());
	assert(!is.bad());
	assert(!is.fail());
	assert(!is.eof());

	int n;
	while (is >> n)
	{
	    cout << "status = " << is.good() << is.bad() << is.fail() << is.eof() << endl;
	    assert(is.good());
	    assert(!is.bad());
	    assert(!is.fail());
	    assert(!is.eof());
	}
	cout << "status = " << is.good() << is.bad() << is.fail() << is.eof() << endl;

	assert(!is.good());
	assert(!is.bad());
	assert(is.fail());
	assert(!is.eof());
	assert(!is);
	assert((void*)is == 0);

	cout << endl;
    }

    /*** Normal ifstream (for reference): non-existant file ***/
    {
        cout << "ifstream: non-existant file" << endl;

        ifstream is("nonexistant.dat");

	cout << "status = " <<is.good() << is.bad() << is.fail() << is.eof() << endl;
	assert(!is.good());
	assert(is.bad());
	assert(is.fail());
	assert(!is.eof());

        cout << endl;
    }

    /*** Test Cistream: normal read to EOF ***/
    {
        cout << "Cistream: normal read to EOF" << endl;

        ifstream iss("testCistream.dat");
	Cistream is(iss.rdbuf());

	cout << "status = " <<is.good() << is.bad() << is.fail() << is.eof() << endl;
	assert(is.good());
	assert(!is.bad());
	assert(!is.fail());
	assert(!is.eof());
	assert(is.lineNumber() == 0);

	string n;
	while (is >> n)
	{
 	    cout << "line " << is.lineNumber() << ": " << n << endl;
	    cout << "status = " <<is.good() << is.bad() << is.fail() << is.eof() << endl;
	    assert(is.good());
	    assert(!is.bad());
	    assert(!is.fail());
	    assert(!is.eof());
	}
	cout << "status = " <<is.good() << is.bad() << is.fail() << is.eof() << endl;

	assert(!is.good());
	assert(!is.bad());
	assert(is.fail());
	assert(is.eof());
	assert(!is);
	assert((void*)is == 0);
 	assert(is.lineNumber() == 5);

	cout << endl;
    }

    /*** Test Cistream: read with error ***/
    {
        cout << "Cistream: read with error" << endl;

        ifstream iss("testCistream.dat");
	Cistream is(iss.rdbuf());

	cout << "status = " << is.good() << is.bad() << is.fail() << is.eof() << endl;
	assert(is.good());
	assert(!is.bad());
	assert(!is.fail());
	assert(!is.eof());
	assert(is.lineNumber() == 0);

	int n;
	while (is >> n)
	{
 	    cout << "line " << is.lineNumber() << ": " << n << endl;
	    cout << "status = " << is.good() << is.bad() << is.fail() << is.eof() << endl;
	    assert(is.good());
	    assert(!is.bad());
	    assert(!is.fail());
	    assert(!is.eof());
	}
	cout << "status = " << is.good() << is.bad() << is.fail() << is.eof() << endl;

	assert(!is.good());
	assert(!is.bad());
	assert(is.fail());
	assert(!is.eof());
	assert(is.lineNumber() == 4);
	assert(!is);
	assert((void*)is == 0);
	assert(is.line() == string("xyz"));

	cout << endl;
    }


    /*** Test Cifstream: normal read to EOF ***/
    {
        cout << "Cifstream: normal read to EOF" << endl;

        Cifstream is("testCistream.dat");

	cout << "status = " <<is.good() << is.bad() << is.fail() << is.eof() << endl;
	assert(is.good());
	assert(!is.bad());
	assert(!is.fail());
	assert(!is.eof());
	assert(is.lineNumber() == 0);

	string n;
	while (is >> n)
	{
	    cout << "line " << is.lineNumber() << ": " << n << endl;
	    cout << "status = " <<is.good() << is.bad() << is.fail() << is.eof() << endl;
	    assert(is.good());
	    assert(!is.bad());
	    assert(!is.fail());
	    assert(!is.eof());
	}
	cout << "status = " <<is.good() << is.bad() << is.fail() << is.eof() << endl;

	assert(!is.good());
	assert(!is.bad());
	assert(is.fail());
	assert(is.eof());
	assert(!is);
	assert((void*)is == 0);
	assert(is.lineNumber() == 5);

	cout << endl;
    }

    /*** Test Cifstream:  read with error ***/
    {
        cout << "Cifstream: read with error" << endl;

        Cifstream is("testCistream.dat");

	cout << "status = " <<is.good() << is.bad() << is.fail() << is.eof() << endl;
	assert(is.good());
	assert(!is.bad());
	assert(!is.fail());
	assert(!is.eof());
	assert(is.lineNumber() == 0);

	int n;
	while (is >> n)
	{
	    cout << "line " << is.lineNumber() << ": " << n << endl;
	    cout << "status = " <<is.good() << is.bad() << is.fail() << is.eof() << endl;
	    assert(is.good());
	    assert(!is.bad());
	    assert(!is.fail());
	    assert(!is.eof());
	}
	cout << "status = " <<is.good() << is.bad() << is.fail() << is.eof() << endl;

	assert(!is.good());
	assert(!is.bad());
	assert(is.fail());
	assert(!is.eof());
	assert(!is);
	assert((void*)is == 0);
	assert(is.lineNumber() == 4);
	assert(is.line() == string("xyz"));

        cout << endl;
    }
  
    /*** Test Cifstream: non-existant file ***/
    {
        cout << "Cifstream: non-existant file" << endl;

        Cifstream is("nonexistant.dat");

	cout << "status = " <<is.good() << is.bad() << is.fail() << is.eof() << endl;
	assert(!is.good());
	assert(is.bad());
	assert(is.fail());
	assert(!is.eof());

        cout << endl;
    }

    /*** Test Cistrstream: normal read to EOF ***/
    {
        cout << "Cistrstream: normal read to EOF" << endl;

        Cistrstream is("#line 1\n"
		       "34 #line 2\n"
		       "45\n"
		       "xyz\n"
		       "ab cd\n");

	cout << "status = " <<is.good() << is.bad() << is.fail() << is.eof() << endl;
	assert(is.good());
	assert(!is.bad());
	assert(!is.fail());
	assert(!is.eof());
	assert(is.lineNumber() == 0);
	
	string n;
	while (is >> n)
	{
 	    cout << "line " << is.lineNumber() << ": " << n << endl;
	    cout << "status = " <<is.good() << is.bad() << is.fail() << is.eof() << endl;
	    assert(is.good());
	    assert(!is.bad());
	    assert(!is.fail());
	    assert(!is.eof());
	}
	cout << "status = " <<is.good() << is.bad() << is.fail() << is.eof() << endl;

	assert(!is.good());
	assert(!is.bad());
	assert(is.fail());
	assert(is.eof());
	assert(!is);
	assert((void*)is == 0);
 	assert(is.lineNumber() == 5);

	cout << endl;
    }

    /*** Test Cistrstream: read with error ***/
    {
        cout << "Cistrstream: read with error   " << endl;

        Cistrstream is("#line 1\n"
		       "34 #line 2\n"
		       "45\n"
		       "xyz\n"
		       "ab cd\n");

	cout << "status = " <<is.good() << is.bad() << is.fail() << is.eof() << endl;
	assert(is.good());
	assert(!is.bad());
	assert(!is.fail());
	assert(!is.eof());
	assert(is.lineNumber() == 0);
	
	int n;
	while (is >> n)
	{
 	    cout << "line " << is.lineNumber() << ": " << n << endl;
	    cout << "status = " <<is.good() << is.bad() << is.fail() << is.eof() << endl;
	    assert(is.good());
	    assert(!is.bad());
	    assert(!is.fail());
	    assert(!is.eof());
	}
	cout << "status = " <<is.good() << is.bad() << is.fail() << is.eof() << endl;

	assert(!is.good());
	assert(!is.bad());
	assert(is.fail());
	assert(!is.eof());
	assert(!is);
	assert((void*)is == 0);
	assert(is.lineNumber() == 4);
	assert(is.line() == string("xyz"));

	cout << endl;
    }

    return 0;
}
